/***********************************************************************************
* INCLUDES
*/
#include "hal_int.h"
#include "hal_timer_32k.h"
#include "hal_board.h"
#include "hal_rf.h"
#include "hal_assert.h"
#include "basic_rf.h"
#include "application.h"
#include "CC2530_UART.h"
#include "uart_queue.h"
#include "hal_rf_util.h"
#include "command.h"

/***********************************************************************************
* LOCAL VARIABLES
*/

static uint8 get_command_data(uint8 * command_p, uint8 * data1_p,
                                     uint8 * data2_p, uint8 * checksum_p)
{
    if(TRUE == UartQueue_Rx_Is_Empty())
    {
        Uart_Send_String(g_UartSerialDev, "[!!ERROR!!] command count fail 1\n");
        return FAILED;
    }
    *command_p = UartQueue_Rx_DeQueue();
    halMcuWaitMs(20);

    if(TRUE == UartQueue_Rx_Is_Empty())
    {
        Uart_Send_String(g_UartSerialDev, "[!!ERROR!!] command count fail 2\n");
        return FAILED;
    }
    *data1_p = UartQueue_Rx_DeQueue();
    halMcuWaitMs(20);

    if(TRUE == UartQueue_Rx_Is_Empty())
    {
        Uart_Send_String(g_UartSerialDev, "[!!ERROR!!] command count fail 3\n");
        return FAILED;
    }
    *data2_p = UartQueue_Rx_DeQueue();
    halMcuWaitMs(20);

    if(TRUE == UartQueue_Rx_Is_Empty())
    {
        Uart_Send_String(g_UartSerialDev, "[!!ERROR!!] command count fail 4\n");
        return FAILED;
    }
    *checksum_p = UartQueue_Rx_DeQueue();

    if(*checksum_p != (uint8)(*command_p + *data1_p + *data2_p))
    {
        Uart_Send_String(g_UartSerialDev, "[!!ERROR!!] checksum fail\n");
        return FAILED;
    }

    return SUCCESS;
}

static void send_response(uint8 command, uint8 data1, uint8 data2)
{
    uint8 checksum;

    UartQueue_Tx_EnQueue(command);
    UartQueue_Tx_EnQueue(data1);
    UartQueue_Tx_EnQueue(data2);

    checksum = command + data1 + data2;
    UartQueue_Tx_EnQueue(checksum);        
}

void appCommandProcess(void)
{
    uint8 command, data1, data2, checksum = 0;

    if(FAILED == get_command_data(&command, &data1, &data2, &checksum))
    {
        send_response(UART_CMD_ERROR, 0x00, 0x00);
        return;
    }

    switch(command)
    {
    case UART_CMD_SET_CHANNEL:
        g_basicRfConfig.channel = data1;
        break;

    case UART_CMD_GET_CHANNEL:
        send_response(command, g_basicRfConfig.channel, 0x00);
        break;

    case UART_CMD_SET_TXPOWER:
#if INCLUDE_PA==2591  || INCLUDE_PA==2592
        if(data1 <= HAL_RF_TXPOWER_20_DBM)
#else
        if(data1 <= HAL_RF_TXPOWER_4_DBM)
#endif
        {
            g_txPower = data1;
        }
        else
        {
            send_response(UART_CMD_ERROR, 0x00, 0x00);
        }
        break;

    case UART_CMD_GET_TXPOWER:
        data1 = g_txPower;
        send_response(command, data1, 0x00);
        break;

    case UART_CMD_SET_MYSHORTID:
        g_basicRfConfig.myAddr = (((uint16)data1<<8) | (uint16)data2);
        break;

    case UART_CMD_GET_MYSHORTID:
        send_response(command,
                      (uint8)(g_basicRfConfig.myAddr>>8), (uint8)(g_basicRfConfig.myAddr));
        break;

    case UART_CMD_SET_DEVID:
        g_deviceAddr = (((uint16)data1<<8) | (uint16)data2);
        break;

    case UART_CMD_GET_DEVID:
        send_response(command,
                      (uint8)(g_deviceAddr>>8), (uint8)g_deviceAddr);
        break;

    case UART_CMD_GET_CHANNEL_SCAN:
    case UART_CMD_GET_RSSI:
        {
            int8  rssiVal;
            int16 rssiRetVal;

            if(UART_CMD_GET_CHANNEL_SCAN == command){
                halSetRxScanMode();
                rssiVal = halSampleED(g_basicRfConfig.channel, 0xFFFF); // 65535 us scan

                if(rssiVal < 128){
                    rssiRetVal = rssiVal - halRfGetRssiOffset();
                }
                else{
                    rssiRetVal = (rssiVal - 256) - halRfGetRssiOffset();
                }                
            }else{
                rssiVal = basicRfGetRssi();
            }

            send_response(command,
                          (uint8)(rssiRetVal>>8), (uint8)rssiRetVal);
        }
        break;
    }
}

