/*-----------------------------------------------------------------------------
|   File:      CC2530_UART.h
+----------------------------------------------------------------------------*/

/*==== DECLARATION CONTROL ==================================================*/

#ifndef CC2530_UART_H
#define CC2530_UART_H

/*==== INCLUDES ==============================================================*/

/*==== CONSTS ================================================================*/

/*==== TYPES =================================================================*/

typedef enum
{
    UART_DEV_UART0,
    UART_DEV_UART1
} UartDevE;

/*==== EXPORTS ===============================================================*/

/*==== MACROS=================================================================*/

// UxCSR (0xF8) - USART x Control and Status - bit accessible SFR register
#define UART_CSR_MODE                        0x80
#define UART_CSR_RE                          0x40
#define UART_CSR_SLAVE                       0x20
#define UART_CSR_FE                          0x10
#define UART_CSR_ERR                         0x08
#define UART_CSR_RX_BYTE                     0x04
#define UART_CSR_TX_BYTE                     0x02
#define UART_CSR_ACTIVE                      0x01

// UxUCR (0xFB) - USART x UART Control
#define UART_1UCR_FLUSH                       0x80
#define UART_UCR_FLOW                        0x40
#define UART_UCR_D9                          0x20
#define UART_UCR_BIT9                        0x10
#define UART_UCR_PARITY                      0x08
#define UART_UCR_SPB                         0x04
#define UART_UCR_STOP                        0x02
#define UART_UCR_START                       0x01

// UxGCR (0xFC) - USART x Generic Control
#define UART_GCR_CPOL                        0x80
#define UART_GCR_CPHA                        0x40
#define UART_GCR_ORDER                       0x20
#define UART_GCR_BAUD_E                      0x1F
#define UART_GCR_BAUD_E0                     0x01
#define UART_GCR_BAUD_E1                     0x02
#define UART_GCR_BAUD_E2                     0x04
#define UART_GCR_BAUD_E3                     0x08
#define UART_GCR_BAUD_E4                     0x10

// PERCFG (0xF1) - Peripheral Control
#define PERCFG_T1CFG                      0x40
#define PERCFG_T3CFG                      0x20
#define PERCFG_T4CFG                      0x10
#define PERCFG_U1CFG                      0x02
#define PERCFG_U0CFG                      0x01

/*==== GLOBALS ================================================================*/

/*==== GLOBAL VARS ============================================================*/

// UART Serial Communction Device ...
extern UartDevE  g_UartSerialDev;
extern UartDevE  g_UartCommDev;

/*==== FUNCTIONS ==============================================================*/

void Uart0_Init(void);
void Uart1_Init(void);

uint8 Uart_Is_Active(UartDevE uartDev);
void Uart_Send_Char_NoWait(UartDevE uartDev, uint8 txChar);

void Uart_Send_Char(UartDevE uartDev, char txChar);
char Uart_Receive_Char(UartDevE uartDev);
void Uart_Send_String(UartDevE uartDev, char * txChar_p);
void Uart_Send_HexVal (UartDevE uartDev, unsigned int hexVal);
void Uart_Send_StringAndHexVal(UartDevE uartDev, char * txChar_p, unsigned int hexVal);

#endif  /* CC2530_UART_H */

/*==== END OF FILE ==========================================================*/
